<?php

/**
 * Replace the default menu fieldset with our menu browser
 *
 * @param array $form
 *  Drupal form array
 * @param array $form_state
 *  Drupal form state array
 */
function menuux_node_form_alter(&$form, &$form_state) {
  if (!isset($form['menu'])) {
    return;
  }
  menuux_mark_items_to_be_hidden($form);

  $node = &$form['#node'];
  $form['menu']['menuux']['#tree'] = true;
  $form['menu']['menuux']['above'] = array(
    '#type' => 'hidden',
    '#default_value' => $node->menu['menuux']['above'],
  );
  if($node->menu['has_children'] > 0) {
    $form['menu']['menuux']['has_children'] = array(
      '#type' => 'hidden',
      '#default_value' => 1
    );
  }
  $form['menu']['#theme'] = 'menuux_node_form';

  /*
   * We store old values to enable us to check if the menu item
   * has been moved. If it hasn't been moved we don't want
   * to change any weights.
   */
  $form['menu']['menuux']['old_above'] = array(
    '#type' => 'value',
    '#value' => $node->menu['menuux']['above'],
  );
  $form['menu']['menuux']['old_parent'] = array(
    '#type' => 'value',
    '#value' => $form['menu']['parent']['#default_value'],
  );
}

/**
 * Mark form items that are to be hidden with a class
 *
 * @param array $form
 *  FAPI node form
 */
function menuux_mark_items_to_be_hidden(&$form) {
  $items_to_be_hidden = array('delete', 'link_title', 'parent', 'weight');
  foreach ($items_to_be_hidden as $item) {
    if (isset($form['menu'][$item]['#attributes']['class'])) {
      $form['menu'][$item]['#attributes']['class'] .= ' menuux-hide-me';
    }
    else {
      $form['menu'][$item]['#attributes']['class'] = 'menuux-hide-me';
    }
  }
}

/**
 * Validate the menu-browser part of nodes beeing created.
 *
 * @param object $node
 *  The node object for the node beeing edited/created
 */
function menuux_node_form_validate(&$node) {
  $illegal = FALSE;
  if ($node->menu['menuux']['above']) {
    if ($node->menu['menuux']['above'] != 'top') {
      if (!is_numeric($node->menu['menuux']['above']) || intval($node->menu['menuux']['above']) != $node->menu['menuux']['above']) {
        $illegal = TRUE;
      }
      $parent_as_array = explode(':', $node->menu['parent']);
      $menu_name_test = db_result(db_query(
        "SELECT 1 FROM {menu_links}
        WHERE mlid = %d AND menu_name = '%s'",
        $node->menu['menuux']['above'], $parent_as_array[0]));
      if (!$menu_name_test) {
        $illegal = TRUE;
      }
    }
    if (empty($node->menu['link_title'])) {
      form_set_error('', t("The menu item must be given a title. If you don't want to add your content to a menu please remove it from the currently selected menu.")
      . '<div id="menuux-title-error"></div>');
    }
  }
  if ($illegal) {
    form_set_error('', t('An illegal choice has been detected. Please contact the site administrator.'));
  }
}

/**
 * Handle the menubrowser part of nodes being inserted or updated
 *
 * @param object $node
 *  The node beeing inserted/updated
 */
function menuux_node_form_presave(&$node) {
  if ($node->menu['parent'] == $node->menu['menuux']['old_parent']
    && $node->menu['menuux']['above'] == $node->menu['menuux']['old_above']) {
    return;
  }
  if (!empty($node->menu['menuux']['above'])) {
    if ($node->menu['menuux']['above'] == 'top') {
      $node->menu['weight'] = menuux_get_minimum_weight($node->menu) - 1;
    }
    else {
      $related_link = db_fetch_array(db_query('SELECT * FROM {menu_links} WHERE mlid = %d', $node->menu['menuux']['above']));
      $node->menu['weight'] = $related_link['weight'] + 1;
      menuux_fix_menu_weights($related_link, 'below');
    }
  }
}

/**
 * Fint the mimum weight for the current menu item and its siblings
 *
 * @param array $menu
 *  Menu as represented in a node form
 * @return int $minimum_weight
 *  The minimum weight for the current menu item and its siblings
 */
function menuux_get_minimum_weight($menu) {
  $parent = explode(':', $menu['parent']);
  return db_result(db_query(
    "SELECT MIN(weight)
    FROM {menu_links}
    WHERE menu_name = '%s' AND plid = %d"
    , $parent[0], $parent[1]
  ));
}

/**
 * Make sure weight are correct when a new item is inserted between existing ones.
 *
 * @param array $link_above
 *  The link we want to place a new item below
 */
function menuux_fix_menu_weights($link_above) {
  $same_as_related = db_result(db_query_range(
    "SELECT 1 FROM {menu_links} WHERE menu_name = '%s' AND weight = %d AND plid = %d AND BINARY link_title > '%s'",
    $link_above['menu_name'], $link_above['weight'], $link_above['plid'], $link_above['link_title'], 0, 1
  ));
  // Several other links has the same weight as the one we want to be placed below...
  if ($same_as_related) {
    db_query(
      "UPDATE {menu_links} SET weight = weight + 2 WHERE menu_name = '%s' AND plid = %d AND ((weight = %d AND BINARY link_title > '%s') OR (weight > %d))",
      $link_above['menu_name'], $link_above['plid'], $link_above['weight'],  $link_above['link_title'], $link_above['weight']
    );
  }
  else {
    $same_as_new = db_result(db_query_range(
      "SELECT 1 FROM {menu_links} WHERE menu_name = '%s' AND weight = %d AND plid = %d",
      $link_above['menu_name'], $link_above['weight'] + 1, $link_above['plid'], 0, 1
    ));
    // Several other links has the same weight as the one we want to be placed below...
    if ($same_as_new) {
      db_query(
        "UPDATE {menu_links} SET weight = weight + 1 WHERE menu_name = '%s' AND weight > %d AND plid = %d",
        $link_above['menu_name'], $link_above['weight'], $link_above['plid']
      );
    }
  }
}

/**
 * Add required javascript and css to the node forms
 *
 * @param <type> $form
 * @return <type>
 */
function theme_menuux_node_form($form) {
  if (user_access('administer menu')) {
    $p = drupal_get_path('module', 'menuux');
    drupal_add_js($p . '/js/menu.js');
    drupal_add_css($p . '/css/menu.css');
  }
  return drupal_render($form);
}

/**
 * find which item is above the loaded item when editing
 *
 * @param array $menu The menu array in the node
 */
function menuux_node_get_above($menu) {
  // verify that link title is set (editing an item)
  if (!$menu['link_title']) {
    // if not return false
    return false;
  }
  else {
    $result = db_query(
     "SELECT mlid, router_path, link_path
      FROM {menu_links}
      WHERE menu_name = '%s'
      AND plid = %d
      AND hidden >= 0
      AND ((weight = %d
      AND BINARY link_title < '%s') OR (weight < %d))
      ORDER BY weight DESC, BINARY link_title DESC",
      $menu['menu_name'], $menu['plid'], $menu['weight'], $menu['link_title'], $menu['weight']
    );
    // Make sure we return a page which the user has access to.
    while ($res_o = db_fetch_object($result)) {
      if ($res_o->router_path != 'node/%') {
        $item = menu_get_item($res_o->link_path);
        if ($item['access']) {
          return $res_o->mlid;
        }
      }
      // We use a faster way to check node access..
      else {
        $nid = substr($res_o->link_path, 5);
        if (is_numeric($nid)) {
          $access = db_result(db_query(db_rewrite_sql(
           "SELECT 1
            FROM {node} n
            WHERE n.status = 1 AND n.nid = %d"), $nid));
          if ($access) {
            return $res_o->mlid;
          }
        }
      }
    }
    return 0;
  }
}